@@ -1,6 +1,6 @@
 /*
  * xinstall.c
- * $Id: xinstall.c 28669 2007-09-06 13:00:25Z afb@macports.org $
+ * $Id: xinstall.c 57602 2009-09-14 01:22:38Z raimue@macports.org $
  * Copyright (c) 1987, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
@@ -70,6 +70,8 @@
 
 #include <tcl.h>
 
+#include "Pextlib.h"
+
 #if HAVE_PATHS_H
 #include <paths.h>
 #endif
@@ -78,6 +80,8 @@
 #define _PATH_DEVNULL	"/dev/null"
 #endif
 
+#include "xinstall.h"
+
 #ifndef MAXBSIZE
 #define MAXBSIZE 65536
 #endif
@@ -137,8 +141,6 @@ static void	strip(const char *);
 static int	trymmap(int);
 static void	usage(Tcl_Interp *interp);
 
-extern int	ui_info(Tcl_Interp *interp, char *mesg);
-
 int
 InstallCmd(ClientData clientData UNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
@@ -434,7 +436,9 @@ install(Tcl_Interp *interp, const char *from_name, const char *to_name, u_long f
 	int devnull, files_match, from_fd = 0, serrno, target;
 	int tempcopy, temp_fd, to_fd = 0;
 	char backup[MAXPATHLEN], *p, pathbuf[MAXPATHLEN], tempfile[MAXPATHLEN];
-	char msg[256];
+
+        /* message contains function name, two paths and a little bit extra formatting */
+        char msg[MAXPATHLEN * 2 + 32];
 
 	files_match = 0;
 
@@ -749,7 +753,7 @@ install(Tcl_Interp *interp, const char *from_name, const char *to_name, u_long f
 	 * flags, except for the dump flag.
 	 * NFS does not support flags.  Ignore EOPNOTSUPP flags if we're just
 	 * trying to turn off UF_NODUMP.  If we're trying to set real flags,
-	 * then warn if the the fs doesn't support it, otherwise fail.
+	 * then warn if the fs doesn't support it, otherwise fail.
 	 */
 #if defined(UF_NODUMP)
 	if (!devnull && (flags & SETFLAGS ||
