@@ -5,20 +5,23 @@ exec @TCLSH@ "$0" "$@"
 
 # Traverse through all ports, creating an index and archiving port directories
 # if requested
-# $Id: portindex.tcl 54378 2009-07-25 21:31:51Z jmr@macports.org $
+# $Id: portindex.tcl 68414 2010-06-03 03:32:33Z jmr@macports.org $
 
 catch {source \
-    [file join "@TCL_PACKAGE_DIR@" macports1.0 macports_fastload.tcl]}
+    [file join "@macports_tcl_dir@" macports1.0 macports_fastload.tcl]}
 package require macports
 package require Pextlib
 
 # Globals
 set archive 0
+set full_reindex 0
 set stats(total) 0
 set stats(failed) 0
+set stats(skipped) 0
 array set ui_options        [list]
 array set global_options    [list]
 array set global_variations [list]
+set port_options            [list]
 
 # Pass global options into mportinit
 mportinit ui_options global_options global_variations
@@ -26,19 +29,48 @@ mportinit ui_options global_options global_variations
 # Standard procedures
 proc print_usage args {
     global argv0
-    puts "Usage: $argv0 \[-ad\] \[-o output directory\] \[directory\]"
+    puts "Usage: $argv0 \[-adf\] \[-p plat_ver_arch\] \[-o output directory\] \[directory\]"
     puts "-a:\tArchive port directories (for remote sites). Requires -o option"
     puts "-o:\tOutput all files to specified directory"
     puts "-d:\tOutput debugging information"
+    puts "-f:\tDo a full re-index instead of updating"
+    puts "-p:\tPretend to be on another platform"
 }
 
 proc pindex {portdir} {
-    global target fd directory archive outdir stats
+    global target oldfd oldmtime newest qindex fd directory archive outdir stats full_reindex \
+           ui_options port_options save_prefix keepkeys
+
+    # try to reuse the existing entry if it's still valid
+    if {$full_reindex != "1" && $archive != "1" && [info exists qindex([string tolower [file tail $portdir]])]} {
+        try {
+            set mtime [file mtime [file join $directory $portdir Portfile]]
+            if {$oldmtime >= $mtime} {
+                set offset $qindex([string tolower [file tail $portdir]])
+                seek $oldfd $offset
+                gets $oldfd line
+                set name [lindex $line 0]
+                set len [lindex $line 1]
+                set line [read $oldfd $len]
+
+                if {[info exists ui_options(ports_debug)]} {
+                    puts "Reusing existing entry for $portdir"
+                }
+
+                puts $fd [list $name $len]
+                puts -nonewline $fd $line
+
+                incr stats(skipped)
+                return
+            }
+        } catch {*} {
+            ui_warn "failed to open old entry for ${portdir}, making a new one"
+        }
+    }
+
     incr stats(total)
-    global macports::prefix
-    set save_prefix $prefix
     set prefix {\${prefix}}
-    if {[catch {set interp [mportopen file://[file join $directory $portdir]]} result]} {
+    if {[catch {set interp [mportopen file://[file join $directory $portdir] $port_options]} result]} {
         puts stderr "Failed to parse file $portdir/Portfile: $result"
         # revert the prefix.
         set prefix $save_prefix
@@ -68,9 +100,8 @@ proc pindex {portdir} {
             }
         }
 
-        set keepkeys {categories depends_fetch depends_extract depends_build depends_lib depends_run description epoch homepage long_description maintainers name platforms revision variants version portdir portarchive replaced_by}
         foreach availkey [array names portinfo] {
-            if {[lsearch -exact ${keepkeys} $availkey] == -1} {
+            if {![info exists keepkeys($availkey)]} {
                 unset portinfo($availkey)
             }
         }
@@ -78,6 +109,10 @@ proc pindex {portdir} {
         set len [expr [string length $output] + 1]
         puts $fd [list $portinfo(name) $len]
         puts $fd $output
+        set mtime [file mtime [file join $directory $portdir Portfile]]
+        if {$mtime > $newest} {
+            set newest $mtime
+        }
     }
 }
 
@@ -96,7 +131,16 @@ for {set i 0} {$i < $argc} {incr i} {
                 set ui_options(ports_debug) yes
             } elseif {$arg == "-o"} { # Set output directory
                 incr i
-                set outdir [lindex $argv $i]
+                set outdir [file join [pwd] [lindex $argv $i]]
+            } elseif {$arg == "-p"} { # Set platform
+                incr i
+                set platlist [split [lindex $argv $i] _]
+                set os_platform [lindex $platlist 0]
+                set os_major [lindex $platlist 1]
+                set os_arch [lindex $platlist 2]
+                lappend port_options os.platform $os_platform os.major $os_major os.arch $os_arch
+            } elseif {$arg == "-f"} { # Completely rebuild index
+                set full_reindex 1
             } else {
                 puts stderr "Unknown option: $arg"
                 print_usage
@@ -104,7 +148,7 @@ for {set i 0} {$i < $argc} {incr i} {
             }
         }
         default {
-            set directory $arg
+            set directory [file join [pwd] $arg]
         }
     }
 }
@@ -143,13 +187,42 @@ if {[info exists outdir]} {
     set outdir $directory
 }
 
-puts "Creating software index in $outdir"
+puts "Creating port index in $outdir"
+set outpath [file join $outdir PortIndex]
+# open old index for comparison
+if {[file isfile $outpath] && [file isfile ${outpath}.quick]} {
+    set oldmtime [file mtime $outpath]
+    set newest $oldmtime
+    if {![catch {set oldfd [open $outpath r]}] && ![catch {set quickfd [open ${outpath}.quick r]}]} {
+        if {![catch {set quicklist [read $quickfd]}]} {
+            foreach entry [split $quicklist "\n"] {
+                set qindex([lindex $entry 0]) [lindex $entry 1]
+            }
+        }
+        close $quickfd
+    }
+} else {
+    set newest 0
+}
+
 set tempportindex [mktemp "/tmp/mports.portindex.XXXXXXXX"]
 set fd [open $tempportindex w]
+set save_prefix ${macports::prefix}
+foreach key {categories depends_fetch depends_extract depends_build \
+             depends_lib depends_run description epoch homepage \
+             long_description maintainers name platforms revision variants \
+             version portdir portarchive replaced_by license} {
+    set keepkeys($key) 1
+}
 mporttraverse pindex $directory
+if {[info exists oldfd]} {
+    close $oldfd
+}
 close $fd
-file rename -force $tempportindex [file join $outdir PortIndex]
-mports_generate_quickindex [file join $outdir PortIndex]
+file rename -force $tempportindex $outpath
+file mtime $outpath $newest
+mports_generate_quickindex $outpath
 puts "\nTotal number of ports parsed:\t$stats(total)\
-      \nPorts successfully parsed:\t[expr $stats(total) - $stats(failed)]\t\
-      \nPorts failed:\t\t\t$stats(failed)\n"
+      \nPorts successfully parsed:\t[expr $stats(total) - $stats(failed)]\
+      \nPorts failed:\t\t\t$stats(failed)\
+      \nUp-to-date ports skipped:\t$stats(skipped)\n"
